import {
  ApolloClient,
  ApolloLink,
  HttpLink,
  InMemoryCache,
  split,
  ApolloProvider,
} from "@apollo/client";
import { withScalars } from "apollo-link-scalars";
import { buildClientSchema, IntrospectionQuery } from "graphql";
import { DateTimeResolver } from "graphql-scalars";
import { FC } from "react";
import introspectionResult from "../../generated/graphql.schema.json";
import { onError } from "@apollo/client/link/error";
import { WebSocketLink } from "@apollo/client/link/ws";
import { getMainDefinition } from "@apollo/client/utilities";
import { useSnackbar } from "notistack";

const schema = buildClientSchema(
  introspectionResult as unknown as IntrospectionQuery
);

const typesMap = {
  DateTime: DateTimeResolver,
};

export const FennecApolloClientProvider: FC = ({ children }) => {
  const { enqueueSnackbar } = useSnackbar();
  const errorLink = onError(({ graphQLErrors, networkError }) => {
    if (graphQLErrors) {
      graphQLErrors.forEach((error) => {
        enqueueSnackbar(error.message, {
          variant: "error",
        });
      });
      graphQLErrors.forEach(({ message, locations, path }) => {
        console.error(
          `[GraphQL error]: Message: ${message}, Location: ${locations}, Path: ${path}`
        );
      });
    }
    if (networkError) {
      console.log(`[Network error]: ${networkError}`);
      enqueueSnackbar(networkError.message, {
        variant: "error",
      });
    }
  });
  const wsLink = new WebSocketLink({
    uri: `${window.location.protocol.replace("http", "ws")}//${
      window.location.hostname
    }:${window.location.port}/api/graphql`,
    options: {
      reconnect: true,
    },
  });
  const httpLink = new HttpLink({
    uri: "/api/graphql",
  });
  const splitLink = split(
    ({ query }) => {
      const definition = getMainDefinition(query);
      return (
        definition.kind === "OperationDefinition" &&
        definition.operation === "subscription"
      );
    },
    wsLink,
    httpLink
  );
  const link = ApolloLink.from([
    errorLink,
    withScalars({ schema, typesMap }) as unknown as ApolloLink,
    splitLink,
  ]);
  const client = new ApolloClient({
    ssrMode: typeof window === "undefined",
    link,
    cache: new InMemoryCache({
      typePolicies: {
        // PipelineTaskLogs: {
        //   keyFields: ["unit"],
        // },
        // PipelineTask: {
        //   fields: {
        //     logs: {
        //       merge(existing = [], incoming: any[]) {
        //         return [...existing, ...incoming];
        //       },
        //     },
        //   },
        // },
      },
    }),
  });

  return <ApolloProvider client={client}>{children}</ApolloProvider>;
};
